#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <SDL.h>
#include <sys/time.h>
#include <flash.h>


FlashHandle flashHandle;
struct FlashInfo fi;
FlashDisplay fd;
struct timeval wd;
bool playing = false;


/* GP2X button mapping */
enum MAP_KEY
{
	VK_UP         , // 0
	VK_UP_LEFT    , // 1
	VK_LEFT       , // 2
	VK_DOWN_LEFT  , // 3
	VK_DOWN       , // 4
	VK_DOWN_RIGHT , // 5
	VK_RIGHT      , // 6
	VK_UP_RIGHT   , // 7
	VK_START      , // 8
	VK_SELECT     , // 9
	VK_FL         , // 10
	VK_FR         , // 11
	VK_FA         , // 12
	VK_FB         , // 13
	VK_FX         , // 14
	VK_FY         , // 15
	VK_VOL_UP     , // 16
	VK_VOL_DOWN   , // 17
	VK_TAT          // 18
};

/* The screen surface, joystick device */
SDL_Surface *screen = NULL;
SDL_Joystick *joy = NULL;


/* Flash part */

unsigned short r_mask = 0;
unsigned short g_mask = 0;
unsigned short b_mask = 0;

unsigned long FlashRGB(unsigned char r, unsigned char g, unsigned char b) {
         return SDL_MapRGB(screen->format,r,g,b);
}

int readFile(char *filename, char **buffer, long *size)
{
	FILE *in;
	char *buf;
	long length;

	in = fopen(filename,"rb");
	if (in == 0) {
		perror(filename);
		return -1;
	}
	fseek(in,0,SEEK_END);
	length = ftell(in);
	rewind(in);
	buf = (char*)malloc(length);
	fread(buf,length,1,in);
	fclose(in);

	*size = length;
	*buffer = buf;

	return length;
}



void InitMask() {
     r_mask = screen->format->Rmask;
     g_mask = screen->format->Gmask;
     b_mask = screen->format->Bmask;
}


char* swfBuffer;
long swfSize;
void InitFlash(char* name) {
     
    readFile(name, &swfBuffer, &swfSize);
    InitMask();
    flashHandle = FlashNew();

	int status;
	int steps = 0;
	do {
		status = FlashParse(flashHandle, 0, (char*)swfBuffer, (int)swfSize);
		steps++;
	} while (status & FLASH_PARSE_NEED_DATA);
    
    free(swfBuffer);
	FlashGetInfo(flashHandle, &fi);

	fd.width = 320;
	fd.height = 240;
	fd.bpl = 320 * 2;
	fd.depth = 16;
	fd.bpp = 2;
	fd.clip_x = 0;
	fd.clip_y = 0;
	fd.clip_width = 320;
	fd.clip_height = 240;
	fd.pixels = screen->pixels;

	status = FlashGraphicInit(flashHandle, &fd);


	FlashSoundInit(flashHandle, "/dev/dsp");
	FlashSettings(flashHandle, PLAYER_LOOP);
     
}

/*  End of flash Part  */




void Terminate(void)
{
	SDL_Quit();
#ifdef GP2X
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
}


int main (int argc, char *argv[])
{
	int done;

	/* Initialize SDL */
	if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0) {
		fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		exit (1);
	}
	atexit (Terminate);

	SDL_ShowCursor(SDL_DISABLE);

	/* Set 320x240 16-bits video mode */
	screen = SDL_SetVideoMode (320, 240, 16, SDL_SWSURFACE);
	if (screen == NULL) {
		fprintf (stderr, "Couldn't set 320x240x16 video mode: %s\n", SDL_GetError ());
		exit (2);
	}

	/* Check and open joystick device */
	if (SDL_NumJoysticks() > 0) {
		joy = SDL_JoystickOpen(0);
		if(!joy) {
			fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
		}
	}

#ifdef GP2X
	/* Only use GP2X code here */
#endif

#ifdef WIN32
	/* Only use Windows code here */
#endif
           
	InitFlash(argv[1]);
	playing = true;
	long cmd, wakeUp;
	cmd = FLASH_REWIND;
	FlashExec(flashHandle, cmd, 0, &wd);
	cmd = FLASH_CONT;
	wakeUp = FlashExec(flashHandle, cmd, 0, &wd);


	done = 0;
	while (!done)
	{
		SDL_Event event;

		/* Check for events */
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					/* if press Ctrl + C, terminate program */
					if ( (event.key.keysym.sym == SDLK_c) && (event.key.keysym.mod & (KMOD_LCTRL | KMOD_RCTRL)) )
						done = 1;
					break;
				case SDL_KEYUP:
					break;
				case SDL_JOYBUTTONDOWN:
					/* if press Start button, terminate program */
					if ( event.jbutton.button == VK_START )
						done = 1;
					break;
				case SDL_JOYBUTTONUP:
					break;
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;
			}
			
			
		}

		/* We can simulate a mouse pointer */
		/*
		FlashEvent fe;
		
		//1
		if(keysPress(VK_FX)) {
			fe.type = FeButtonPress;
			FlashExec(flashHandle,FLASH_EVENT,&fe,&wd);
		}
		
		//2
		if(keysPress(VK_FB)) {
			fe.type = FeButtonRelease;
			FlashExec(flashHandle,FLASH_EVENT,&fe,&wd);
		}
		
		//3
		if(keysPress(VK_UP) || keysPress(VK_DOWN) etc...) 
			fe.type = FeMouseMove;
			fe.x += keysPress(VK_RIGHT)-keysPress(VK_LEFT);
			fe.y += keysPress(VK_DOWN)-keysPress(VK_UP);
			FlashExec(flashHandle,FLASH_EVENT,&fe,&wd);
		}
		
		*/
		
		cmd = FLASH_WAKEUP;
		wakeUp = FlashExec(flashHandle, cmd, 0, &wd);
		
		SDL_Flip(screen);
	}

	return 0;
}
